/******************************************************************************
Filename:     modRfSimpleImpl.c

Description:
H.Zheng 802.15.4 building block layer module library: RF part

һּ򵥵modRf.hӿڵʵַʽ
ֱӷͷ飬ackطܡ

modification:
2008.1.5 creation Zheng Hui
2011.5.30 modified to meet cc2530 usage, base on basicRF
2011.7.3  discard basicRFackط
2012.05.06 ¶Ĺӿ޸ġ㹦ܣMS-LRWPAN stackתƵmodRfStackС
           ļʵ·㹦ܣṩStackĵײ֧֡

COPYRIGHT
Zheng Hui, Guangzhou, China. hzheng@gzhu.edu.cn, Guangzhou University

******************************************************************************/
//#include <string.h>
//#include <stdio.h>
#include <ioCC2530.h>
#include "modRf.h"
//#include "hal_dma.h"
//#include "basic_rf.h"
#include "hal_rf.h"
#include "hal_rf_security.h"
#include "hal_int.h"
#include "hal_mcu.h"
#include "lmodRfStack.h"

//Type Define

//char type in ROM memory
typedef  unsigned char const __code    ROMCHAR ;


typedef struct {
    uint16 myAddr;
    uint16 panId;
    uint8 channel;
    uint8 ackRequest;
    uint8 securityFlag;
    uint8* securityKey;
    uint8* securityNonce;
} modRfCfg_t;

typedef struct {
    uint8 txSeqNumber;
    volatile uint8 ackReceived;
    uint8 receiveOn;
    uint32 frameCounter;
} modRfTxState_t;

//constant
#define FLAG_FIELD                          0x09
#define NONCE_SIZE                          16

// Packet overhead ((frame control field, sequence number, destPANID, destAddr, srcPANID, srcAddr) + (footer))
#define MOD_RF_PACKET_HEADER_SIZE       (2 + 1 + 2 + 2 + 2 + 2 )
#define MOD_RF_FOOTER_SIZE                2
#define MOD_RF_PACKET_OVERHEAD_SIZE       (MOD_RF_PACKET_HEADER_SIZE + MOD_RF_FOOTER_SIZE)
#define MOD_RF_AUX_HDR_LENGTH             5
#define MOD_RF_ACK_PACKET_SIZE	          5
#define MOD_RF_SECURITY_M                 2
#define MOD_RF_LEN_MIC                    8

#define MOD_RF_MAX_PAYLOAD_SIZE	        (127 - MOD_RF_PACKET_OVERHEAD_SIZE - MOD_RF_AUX_HDR_LENGTH - MOD_RF_LEN_MIC)
#define MOD_RF_LEN_AUTH                   MOD_RF_PACKET_OVERHEAD_SIZE + MOD_RF_AUX_HDR_LENGTH - MOD_RF_FOOTER_SIZE

#define AUTO_CRC                   0x40
#define CC2530_TXPOWER_0_DBM       0xD5
#define IRQ_TXDONE                 0x02



//parameter
static modRfCfg_t mRfConfig;
static modRfTxState_t mTxState=  { 0x00 };
static uint8 mRawRxMpdu[130];
static uint8 mTxDataMpdu[130];
//static uint8 mTxAckMpdu[10];
static uint8 mNonceTx[NONCE_SIZE];
static uint8 mNonceRx[NONCE_SIZE];

//callback function
//for tx
static modRfTxCallbackFunction* mTxStartCallBackFunction=NULL;
static modRfTxCallbackFunction* mTxEndCallBackFunction=NULL;

//for receive
static modRfRxRawCallbackFunction* mRxRawCallBackFunction=NULL;
static modRfRxCallbackFunction* mRxCallBackFunction=NULL;
void modRfRxCallBack(void);
static void modRfRxFrmDoneIsr(void);
void modRfSecurityInit(void);
static uint8 modRfBuildMpdu(uint16 vDestAddr, uint8* vPayload, uint8 vPayloadLength);
static void modRfBuildMpduHeader(void);



/**
 * Ƶʼýӿ
 * 
 *     vNodeId: ڵַȡֵΧ0-0xFFFE0x0FFFFΪ㲥ַã
 *     vPanId: IDͬPanIdĽڵſ໥ͨţȡֵΧ0-0xFFFE0x0FFFFݲʹã
 *     vChannel: ƵŵIDȡֵΧ淶أ
 *     vAckRequestFlag: ·Ϣȷϱ־ʾ͵ķǷҪԷȷϢ0ΪҪ0ΪҪ
 *     vSecurityFlag: ·ܱ־ʾ͵ķǷҪܣ0ΪҪ0ΪҪҪʱvKeyΪ0
 *     vKey: ·Կָ룬ָԿԿʵֻأҪʱΪ0
 *
 * ע⣺ʵδ֧Ackͼܣ
 */
int8 modRfInit(uint16 vNodeId, uint16 vPanId, uint8 vChannel, uint8 vAckRequestFlag,
               uint8 vSecurityFlag, uint8 *vKey){

  mRfConfig.myAddr = vNodeId;
  mRfConfig.panId = vPanId;
  mRfConfig.channel = vChannel;
  mRfConfig.ackRequest = vAckRequestFlag;
  mRfConfig.securityFlag = vSecurityFlag;
  if (vSecurityFlag > 0) {
    mRfConfig.securityKey = vKey;
  }

  mTxState.receiveOn = 1;
  mTxState.frameCounter = 0;
  //
  modRfBuildMpduHeader(); //init the header part of mTxDataMpdu


  //halRfInit()
  // Enable auto ack and auto crc
  //FRMCTRL0 |= (AUTO_ACK | AUTO_CRC);
  FRMCTRL0 |= AUTO_CRC;
  TXPOWER = CC2530_TXPOWER_0_DBM;      // TX output power ~0 dBm
  TXFILTCFG = 0x09;
  AGCCTRL1 = 0x15;
  FSCAL1 = 0x00;
  FRMFILT0 = FRMFILT0&0xFE; //clear bit0, disable frame filtering
  //

  halIntOff();


  // Set channel
  halRfSetChannel(vChannel);

  // Write the short address and the PAN ID to the chip
  halRfSetShortAddr(vNodeId);
  halRfSetPanId(vPanId);

  // if security is enabled, write key and nonce
  if (vSecurityFlag > 0) {
    modRfSecurityInit();
  }

  // Set up receive interrupt service handler
  halRfRxInterruptConfig(modRfRxFrmDoneIsr);

  // Enable RX interrupt
  halRfEnableRxInterrupt();

  //
  modRfStartRx();

  //
  halIntOn();

  return 0;
}


/**
 * ͷ
 */
static uint8 modRfBuildMpdu(uint16 vDestAddr, uint8* vPayload, uint8 vPayloadLength) {
  uint8 i,iPktSize;

  for(i=0;i<vPayloadLength;i++)  {
      mTxDataMpdu[1+MOD_RF_PACKET_HEADER_SIZE+i] = vPayload[i];
  }
  //
  iPktSize = 1 + MOD_RF_PACKET_HEADER_SIZE + vPayloadLength + MOD_RF_FOOTER_SIZE;
  mTxDataMpdu[0]=iPktSize-1; //ȥsizeռõһֽ
  mTxDataMpdu[3]= mTxState.txSeqNumber;
  mTxDataMpdu[6]=(vDestAddr>>8)&0xFF;
  mTxDataMpdu[7]=vDestAddr&0xFF;

  return iPktSize; // total mpdu length
}

/**
 * ͷheader
 */
static void modRfBuildMpduHeader() {
  //mTxDataMpdu[0]=pktSize;
  //mTxDataMpdu[1]=0x88; //frame control
  //mTxDataMpdu[2]=0x01;
  mTxDataMpdu[1]=0x01; //frame control
  mTxDataMpdu[2]=0x88;

  //mTxDataMpdu[3]= mSeqNumber;
  mTxDataMpdu[4]=(mRfConfig.panId>>8)&0xFF; //dest PAN ID
  mTxDataMpdu[5]=mRfConfig.panId&0xFF;
  //mTxDataMpdu[6]=(destAddr>>8)&0xFF;
  //mTxDataMpdu[7]=destAddr&0xFF;
  mTxDataMpdu[8]=(mRfConfig.panId>>8)&0xFF;
  mTxDataMpdu[9]=mRfConfig.panId&0xFF;
  mTxDataMpdu[10]=(mRfConfig.myAddr>>8)&0xFF;
  mTxDataMpdu[11]=mRfConfig.myAddr&0xFF;

}

/**
 * security init
 */
void modRfSecurityInit(){
    uint8 i;

    // Initialise nonce bytes to 0
    for(i=0;i<NONCE_SIZE;i++)
    {
        mNonceRx[i] = 0;
        mNonceTx[i] = 0;
    }

    // Set nonce flag field (Byte 0)
    mNonceRx[0] = FLAG_FIELD;
    mNonceTx[0] = FLAG_FIELD;

    // Set byte 7 and 8 of nonce to myAddr
    mNonceTx[8] = (uint8)mRfConfig.myAddr;
    mNonceTx[7] = (uint8)(mRfConfig.myAddr>>8);

    // Set Security mode field of nonces (Byte 13)
    mNonceRx[13] = SECURITY_CONTROL;
    mNonceTx[13] = SECURITY_CONTROL;

    halRfSecurityInit(mRfConfig.securityKey, mNonceRx, mNonceTx);
}



/**
 * ͷӿ
 * 
 *     vDestinationId: Զ˵ַȡֵΧ0-0xFFFF0x0FFFFΪ㲥
 *     vSize: ͵û鳤ȣֽĿȡֵΧԼ·ʵֻйأ
 *     vPacket: ûָ룬ָ͵û飻
 */
int8 modRfSendPkt(uint16 vDestinationId, uint8 vSize, uint8* vPacket) {
  uint8 iMpduLength;
  int8 ret=0;
  uint8 iDelayCount;


  // Turn on receiver if its not on
  if(!mTxState.receiveOn) {
      halRfReceiveOn();
  }

  // Wait until the transceiver is idle
  halRfWaitTransceiverReady();

  // Turn off RX frame done interrupt to avoid interference
  halRfDisableRxInterrupt();

  //build the packet to be sent
  iMpduLength = modRfBuildMpdu(vDestinationId, vPacket, vSize);

  //write packet data to RFD buffer
  /*
  if (mRfConfig.securityFlag>0) {
    halRfWriteTxBufSecure(mTxDataMpdu, iMpduLength, vSize, MOD_RF_LEN_AUTH, MOD_RF_SECURITY_M);
    mTxState.frameCounter++;     // Increment frame counter field
    halRfIncNonceTx();          // Increment nonce value
  }else {
    halRfWriteTxBuf((uint8*)mTxDataMpdu, iMpduLength);
  }
  */
  halRfWriteTxBuf((uint8*)mTxDataMpdu, iMpduLength);


  //send packet

  // Send frame with CCA. return FAILED if not successful
  /*
  if(halRfTransmit() != SUCCESS) {
    ret = FAILED;
  }else {
    ret = SUCCESS;
  }
  */

  //wait for CCA for a specified period
  iDelayCount =100;
  while (!(FSMSTAT1&0x10)) { //FSMSTAT1.4=0
    halMcuWaitUs(100);
    iDelayCount --;
    if (iDelayCount == 0) {
      ret = -1;
      break;
    }
  }
  if (FSMSTAT1&0x10) { //CCA set
    ISTXON(); // Sending

    // Waiting for transmission to finish
    iDelayCount =100;
    while(!(RFIRQF1 & IRQ_TXDONE) ) {
      halMcuWaitUs(100);
      iDelayCount --;
      if (iDelayCount == 0) {
        ret = -1;
        break;
      }
    }

    RFIRQF1 = ~IRQ_TXDONE;
  }



  // Turn off the receiver if it should not continue to be enabled
  if (!mTxState.receiveOn) {
      halRfReceiveOff();
  }

  if(ret == 0) {
      mTxState.txSeqNumber++;
  }

  // Turn on RX frame done interrupt for ACK reception
  halRfEnableRxInterrupt();

  return ret;

}




/**
 * ýշصӿ
 * 
 *     vFunction: ûƵķصָ룻
 */
int8 modRfSetRxCallback(modRfRxCallbackFunction* vFunction ){
  mRxCallBackFunction = vFunction;
  return 0;
}

/**
 * ģʽӿ
 */
int8 modRfStartRx(){
  mTxState.receiveOn = TRUE;
  halRfReceiveOn();
  return 0;
}

/**
 * ͹ýӿ
 * 
 *     vPowerLevel: 书ʵȼ0Ϊ0dBmӦ0dBmӦ0dBmӦϵӲأ
 * CC2530ʵ֣(default =0xF5)
 * vPowerLevel=3: power=4.5dBm; registerValue=0xF5;
 * vPowerLevel=2: power=2.5dBm; registerValue=0xE5;
 * vPowerLevel=1: power=1dBm; registerValue=0xD5;
 * vPowerLevel=0: power=-0.5dBm; registerValue=0xC5;
 * vPowerLevel=-1: power=-3dBm; registerValue=0xA5;
 * vPowerLevel=-2: power=-6dBm; registerValue=0x85;
 * vPowerLevel=-3: power=-12dBm; registerValue=0x55;
 * vPowerLevel=-4: power=-18dBm; registerValue=0x25;
 * vPowerLevel=-5: power=-22dBm; registerValue=0x05;

 */
int8 modRfSetTxPower(int8 vPowerLevel){
  //map vPowerLevel to register value
    switch(vPowerLevel)  {
    case (3):
      TXPOWER=0xF5;
      break;
    case (2):
      TXPOWER=0xE5;
      break;
    case (1):
      TXPOWER=0xD5;
      break;
    case (0):
      TXPOWER=0xC5;
      break;
    case (-1):
      TXPOWER=0xA5;
      break;
    case (-2):
      TXPOWER=0x85;
      break;
    case (-3):
      TXPOWER=0x55;
      break;
    case (-4):
      TXPOWER=0x25;
      break;
    case (-5):
      TXPOWER=0x05;
      break;
    default:
      return MOD_RET_NOT_SUPPORT;
    }

  return 0;
}






/**
 * ͷӿ: LR_WPAN stack
 */
signed char modRfSendRawPkt(unsigned char vSize, unsigned char* vPacket) {
  int8 ret=0;
  uint8 iDelayCount;
  uint8 i;


  // Turn on receiver if its not on
  if(!mTxState.receiveOn) {
      halRfReceiveOn();
  }

  // Wait until the transceiver is idle
  halRfWaitTransceiverReady();

  // Turn off RX frame done interrupt to avoid interference
  halRfDisableRxInterrupt();

  //send packet
  ISFLUSHTX();          // Making sure that the TX FIFO is empty.
  RFIRQF1 = ~IRQ_TXDONE;   // Clear TX done interrupt

  RFD = vSize+2;
  // Insert data
  for(i=0;i<vSize;i++){
    RFD = vPacket[i];
  }



  //wait for CCA for a specified period
  iDelayCount =100;
  while (!(FSMSTAT1&0x10)) { //FSMSTAT1.4=0
    halMcuWaitUs(100);
    iDelayCount --;
    if (iDelayCount == 0) {
      ret = -1;
      break;
    }
  }
  //if (FSMSTAT1&0x10) { //CCA set
  if (ret == 0) { //CCA set
    ISTXON(); // Sending
    //callback
    if(mTxStartCallBackFunction) {
       mTxStartCallBackFunction();
    }


    // Waiting for transmission to finish
    iDelayCount =100;
    while(!(RFIRQF1 & IRQ_TXDONE) ) {
      halMcuWaitUs(100);
      iDelayCount --;
      if (iDelayCount == 0) {
        ret = -1;
        break;
      }
    }
    if (ret == 0) {
      //callback
      if(mTxEndCallBackFunction) {
         mTxEndCallBackFunction();
      }

    }

    RFIRQF1 = ~IRQ_TXDONE;
  }

  // Turn off the receiver if it should not continue to be enabled
  if (!mTxState.receiveOn) {
      halRfReceiveOff();
  }

  if(ret == 0) {
      mTxState.txSeqNumber++;
  }

  // Turn on RX frame done interrupt for ACK reception
  halRfEnableRxInterrupt();

  return ret;

}

/**
 * Ƶŵýӿ
 */
void modSetRfChannel(uint8 vChannel) {
  mRfConfig.channel = vChannel;
  halRfSetChannel(vChannel);
}
/**
 * ڵַýӿ
 */
void modSetRfNodeId(unsigned short vNodeId) {
  mRfConfig.myAddr = vNodeId;
  halRfSetShortAddr(vNodeId);
  modRfBuildMpduHeader();
}

unsigned short modGetRfNodeId() {
  return mRfConfig.myAddr;
}
/**
 * PANַýӿ
 */
void modSetRfPanId(unsigned short vPanId) {
  mRfConfig.panId = vPanId;
  halRfSetPanId(vPanId);
  modRfBuildMpduHeader();
}

unsigned short modGetRfPanId() {
  return mRfConfig.panId;
}


/**
 * ȡFlashеIEEEַӿ
 */
#define CC2430_FLASH_SIZE 128

#if (CC2430_FLASH_SIZE == 32)
#define IEEE_ADDRESS_ARRAY 0x7FF8
#elif (CC2430_FLASH_SIZE == 64) || (CC2430_FLASH_SIZE == 128)
#define IEEE_ADDRESS_ARRAY 0xFFF8
#endif

void modGetProcessorIEEEAddress(unsigned char *vBuf) {
#if (CC2430_FLASH_SIZE == 128)
  unsigned char bank;
  bank = MEMCTR;
  //switch to bank 3
  MEMCTR |= 0x30;
#endif
  //note that the flash programmer stores these in BIG ENDIAN order for some reason!!!
  vBuf[7] = *(ROMCHAR *)(IEEE_ADDRESS_ARRAY+0);
  vBuf[6] = *(ROMCHAR *)(IEEE_ADDRESS_ARRAY+1);
  vBuf[5] = *(ROMCHAR *)(IEEE_ADDRESS_ARRAY+2);
  vBuf[4] = *(ROMCHAR *)(IEEE_ADDRESS_ARRAY+3);
  vBuf[3] = *(ROMCHAR *)(IEEE_ADDRESS_ARRAY+4);
  vBuf[2] = *(ROMCHAR *)(IEEE_ADDRESS_ARRAY+5);
  vBuf[1] = *(ROMCHAR *)(IEEE_ADDRESS_ARRAY+6);
  vBuf[0] = *(ROMCHAR *)(IEEE_ADDRESS_ARRAY+7);
#if (CC2430_FLASH_SIZE == 128)
  //resore old bank settings
  MEMCTR = bank;
#endif
}

/**
 * üĴеIEEEַ
 */
void modSetRfIEEEAddress(unsigned char *vBuf) {
  EXT_ADDR0 = vBuf[0];
  EXT_ADDR1 = vBuf[1];
  EXT_ADDR2 = vBuf[2];
  EXT_ADDR3 = vBuf[3];
  EXT_ADDR4 = vBuf[4];
  EXT_ADDR5 = vBuf[5];
  EXT_ADDR6 = vBuf[6];
  EXT_ADDR7 = vBuf[7];

}


/**
 * ÷ͷصӿ
 * appģڸúʵֶԷĴ
 */

void modRfSetTxStartCallback(modRfTxCallbackFunction* vFunction) {
  mTxStartCallBackFunction = vFunction;
}
void modRfSetTxEndCallback(modRfTxCallbackFunction* vFunction) {
  mTxEndCallBackFunction = vFunction;
}



/**
 * ýշصӿ
 * appģڸúʵֶԷĴ
 */

void modRfSetRxRawCallback(modRfRxRawCallbackFunction* vFunction) {
  mRxRawCallBackFunction = vFunction;
}




/**
 * жϷmodRFƬϣ
 * halRFеĽжϷ
 */
static void modRfRxFrmDoneIsr(void) {
  uint8 iPktLength;
  //uint16 iSrcAddr, iDestAddr, iDestPanId;
  uint16 iSrcAddr, iDestAddr;
  uint8 iRssi8;
  int16 iRssi16;
  uint8 iCrc;

  // Clear interrupt and disable new RX frame done interrupt
  halRfDisableRxInterrupt();

  // Enable all other interrupt sources (enables interrupt nesting)
  //halIntOn();

  //Read payload length
  halRfReadRxBuf(mRawRxMpdu,1);
  iPktLength = mRawRxMpdu[0]&0x7F;
  //read payload
  halRfReadRxBuf(mRawRxMpdu+1, iPktLength);

  //
  if(mRxRawCallBackFunction) {
     mRxRawCallBackFunction(iPktLength, mRawRxMpdu);
  }else {
    //read header fields
    //iDestPanId = (mRawRxMpdu[4]&0xff)*256+(mRawRxMpdu[5]&0xff);
    iDestAddr = (mRawRxMpdu[6]&0xff)*256+(mRawRxMpdu[7]&0xff);
    iSrcAddr = (mRawRxMpdu[10]&0xff)*256+(mRawRxMpdu[11]&0xff);
    //
    iRssi8 = mRawRxMpdu[iPktLength-1];
    if(iRssi8 < 128){
      iRssi16 = iRssi8 - halRfGetRssiOffset();
    } else{
      iRssi16 = (iRssi8 - 256) - halRfGetRssiOffset();
    }
    //
    iCrc=mRawRxMpdu[iPktLength]&0x80;

    //filter
    if (iCrc>0) {
      if((iDestAddr == mRfConfig.myAddr) || (iDestAddr == MOD_BROADCAST_ADDRESS)) { //ĿĵǱڵַ㲥ַ
        if(mRxCallBackFunction) {
           mRxCallBackFunction(iSrcAddr, iDestAddr, iPktLength-11-2, mRawRxMpdu+12, iRssi16); //ûָصûݳȣpktLengthȥheader11ֽڣָʾֽڱԼRSSI2ֽڡ
        }
      }
    }
  }

  // Enable RX frame done interrupt again
  //halIntOff();
  halRfEnableRxInterrupt();
}

