/******************************************************************************
Filename:     modTimer.c
Target: CC2530 of TI

Description:
H.Zheng MCU building block layer module library: Timer part

һֶʱӿڵʵַʽ


modification:
2012.06.20 creation Zheng Hui
2012.07.16 H.Zheng vCallbackIndex should 0
//2014.03.31 H.Zheng change interrupt period from 1ms to 8us
2014.08.25 change it back to 1ms

COPYRIGHT
Zheng Hui, Guangzhou, China. hzheng@gzhu.edu.cn. Guangzhou University

******************************************************************************/
#include <ioCC2530.h>
#include "modTimer.h"

typedef struct {
  uint8 bIsUsedFlag;
  uint16 iNumOfOverflowsCfg;
  modTimerTimeoutCallbackFunction* pCallback;
  uint8 bRepeatFlag;
  uint16 iNumOfOverflowsLeft;
} modTimerCallbackCfg_t;

#define MAX_CALLBACK_FUNCTION_NUM 8

static modTimerCallbackCfg_t mTimerCallbackConfig[MAX_CALLBACK_FUNCTION_NUM];

static uint8 mTimer3IsRunningFlag = 0;
static uint8 mTimerCallbackCount = 0;
/**
 * ͬʱӼֵ
 * Timer3ÿoverflowжһΣ1ms1
 */
uint32 gNetSyncTimerCounter=0;

/**
 * ϵͳʱʼӿ
 * ʱжڣ룩
 */
int8 modTimerInit() {
  //ʼmTimerCallbackConfigݽṹ
  uint8 i;
  for (i=0;i<MAX_CALLBACK_FUNCTION_NUM;i++) {
    mTimerCallbackConfig[i].bIsUsedFlag = 0;
  }
  mTimer3IsRunningFlag = 0;
  mTimerCallbackCount = 0;
  gNetSyncTimerCounter = 0;

  //CC2530: ʹTimer3Ϊʱʱ
  //ϵͳʱΪ32MHzCLKCONCMD.TICKSPD[2:0]ƵTxCTL.DIV[2:0]ѡ111: Tick frequency/128ʱ
  //1freerun overflow1ms
/*
  //ϵͳʱΪ32MHzCLKCONCMD.TICKSPD[2:0]ƵTxCTL.DIV[2:0]ѡ000: Tick frequency/1ʱ
  //1freerun overflow8us
*/
  CLKCONCMD &= 0x0C7; //bit5:3=000, 32MHz
  T3CTL = 0x0E8;//11101000: divide=128; OVFIM=1; FreeRun mode;Start=0
  //T3CTL = 0x88;//10001000: divide=16; OVFIM=1; FreeRun mode;Start=0
  T3CCTL0 = 0; //disable channel 0 function
  T3CCTL1 = 0; //disable channel 1 function
  //
  T3IE = 1;
  //
  return 0;

}

/**
 * Ӷʱصӿ
 * 
 *     vNumOfOverflows: ٴжϵһλص
 *     vFunction: صָ
 *     vRepeatFlag: Ƿظ0ʾظ=0ʾֻصһ
 * 
 *     vCallbackIndex: ʾûصõľȡصãòֵ>0
 */
int8 modTimerAddTimeoutCallback(uint16 vNumOfOverflows, modTimerTimeoutCallbackFunction* vFunction, uint8 vRepeatFlag, uint8* vCallbackIndex ) {
  uint8 i;
  for (i=0;i<MAX_CALLBACK_FUNCTION_NUM;i++) {
    if (mTimerCallbackConfig[i].bIsUsedFlag == 0) {//empty config item
      mTimerCallbackConfig[i].bIsUsedFlag = 1;
      mTimerCallbackConfig[i].iNumOfOverflowsCfg = vNumOfOverflows;
      mTimerCallbackConfig[i].pCallback = vFunction;
      mTimerCallbackConfig[i].bRepeatFlag = vRepeatFlag;
      mTimerCallbackConfig[i].iNumOfOverflowsLeft = vNumOfOverflows;
      *vCallbackIndex = i+1;
      mTimerCallbackCount ++;
      if (mTimer3IsRunningFlag == 0) {
        T3CTL |= 0x10; //start = 1
        EA = 1;
        mTimer3IsRunningFlag = 1;
      }
      return 0;
    }
  }
  return -1;
}

/**
 * ȡʱصӿ
 * 
 *     vCallbackIndex: ʾûصõľmodTimerAddTimeoutCallbackṩӦá
 */
int8 modTimerRemoveTimeoutCallback(uint8 vCallbackIndex) {
  if ((vCallbackIndex ==0 )||(vCallbackIndex>MAX_CALLBACK_FUNCTION_NUM)) {
    return -1;
  }
  if (mTimerCallbackConfig[vCallbackIndex-1].bIsUsedFlag > 0) {
    mTimerCallbackConfig[vCallbackIndex-1].bIsUsedFlag = 0;
    mTimerCallbackCount --;
    if (mTimerCallbackCount ==0) {
      //disable timer
      T3CTL &= 0x0EF; //start = 0
    }
    return 0;
  }
  return -1;

}



/**
 * Timer 3 жϷ
 */
#pragma vector=T3_VECTOR
__interrupt void TIMER3_IRQ(void){
  uint8 i;
  //ж
  EA=0;


  //жжԴ
  //жϱ־
  //T1IF=0; //жϱ־

  if ((T3CTL&0x80)&&(TIMIF&0x01)) { //T3CTL.IM=1  T3OVFIF=1
    //Timer3 overflowжϴ
    gNetSyncTimerCounter++;
    //
    if (mTimer3IsRunningFlag>0) {
      for (i=0;i<MAX_CALLBACK_FUNCTION_NUM;i++) {
        if (mTimerCallbackConfig[i].bIsUsedFlag == 1) {//valid config item
          mTimerCallbackConfig[i].iNumOfOverflowsLeft --;
          if (mTimerCallbackConfig[i].iNumOfOverflowsLeft == 0) { //app timeout
            //run callback
            if (mTimerCallbackConfig[i].pCallback) {
              mTimerCallbackConfig[i].pCallback(i);
            }
            //reset app timeout setting
            if (mTimerCallbackConfig[i].bRepeatFlag == 0) { //run once only
              mTimerCallbackConfig[i].bIsUsedFlag = 0; //remove config
              mTimerCallbackCount --;
              if (mTimerCallbackCount ==0) {
                //disable timer
                T3CTL &= 0x0EF; //start = 0
                break; //break for loop
              }
            }else {
              //reset count value
              mTimerCallbackConfig[i].iNumOfOverflowsLeft = mTimerCallbackConfig[i].iNumOfOverflowsCfg;
            }
          }
        }
      }
    }

    //Timer3 overflowжϴ
  }

  //Timer3жϱ־
  TIMIF &= 0x0F8;
  T3IF = 0;

  //ж
  EA=1;

}


