/******************************************************************************
Filename:     modUartSimpleImpl.h
Target:       cc2430/cc2510
Revised:      2008.04.11
Revision:     0.10

Description:
H.Zheng 802.15.4 building block layer module library: UART part


modification:
2008.4.11 creation Zheng Hui
2008.08.16 add uart0 receive handler
2011.06.05 changed to suitable for CC2530
2011.10.05 add Interrupt handler for RX and TX, support UART0 only
2012.04.24 rearrange
2014.04.06 change rx isr, new packet not cover previous packet
COPYRIGHT
Zheng Hui, Guangzhou, China. hzheng@gzhu.edu.cn. Guangzhou University

******************************************************************************/
#include "modUart.h"
#include "ioCC2530.h"
//#include "hal_int.h"



/**
 * ȫֱ
 */
#define MOD_UART_MAX_PDU_SIZE 120;

//UART global FIFO Buffer
#define UART_TX_BUFFER_LEN 360
unsigned char gUart0TxBuffer[UART_TX_BUFFER_LEN];
uint16 gUart0TxStartPointer=0;
uint16 gUart0TxEndPointer=0;
uint8 gUart0SendingFlag=0;

#define UART_RX_BUFFER_LEN 360
unsigned char gUart0RxBuffer[UART_RX_BUFFER_LEN];
uint16 gUart0RxStartPointer=0;
uint16 gUart0RxEndPointer=0;
uint16 gUart0RxCounter = 0;


static modUartRxCallbackFunction* mUartRxUsrCallBackFunction=0;


/**
 * 
 */
//signed char modUartSendPkt(unsigned char vUartNumber, unsigned char vSize, unsigned char* vPacket);
uint16 modUart0GetTxFreeBufferSize();
int8 modUart0PushTxFIFO(unsigned char vData);
unsigned char modUart0PopTxFIFO();


//void modOnUart0ReceivePkt(unsigned char* vPacket, unsigned char vSize);
uint16 modUart0GetRxFreeBufferSize();
int8 modUart0PushRxFIFO(unsigned char vData);
unsigned char modUart0PopRxFIFO();
//signed char modUart0RxFixSizeData(unsigned char* vPacket, unsigned char vSize);

unsigned char modBaudE(long vBaudrate);
unsigned char modBaudM(long vBaudrate);


/**
 * òʼ
 */
//for simplify

int8 modUartInit(uint8 vUartNumber, uint32 vBaudrate){
  unsigned char pBaudE, pBaudM;
  pBaudE = modBaudE(vBaudrate);
  pBaudM = modBaudM(vBaudrate);
  if ((pBaudE==0)||(pBaudM==0)) {  //unsupported Baudrate
    return -2;
  }
  if (vUartNumber>1) {
    return -3;
  }

  //for simple, did not convert option
  do {
    if(vUartNumber == 0){   /* USART0    */
      if(PERCFG & 0x01){ /* Alt 2    */
        P1SEL |= 0x30;
      } else {      /* Alt 1    */
        P0SEL |= 0x0C;
      }
    }else {        /* USART1    */
      if(PERCFG & 0x02){ /* Alt 2    */
        P1SEL |= 0xC0;
      }else {      /* Alt 1    */
        P0SEL |= 0x30;
      }
    }

    if ( vUartNumber == 0) {
      U0GCR = modBaudE(vBaudrate);
      U0BAUD = modBaudM(vBaudrate);
      U0CSR |= 0x80;

    }else {
      U1GCR = modBaudE(vBaudrate);
      U1BAUD = modBaudM(vBaudrate);
      U1CSR |= 0x80;

    }

  } while(0);

  //enable uart0 interrupt
  UTX0IF = 0;
  IEN2 |= 0x04;
  //
  U0CSR |= 0x40; //set U0CSR.RE
  URX0IF = 0;
  URX0IE = 1;
  //
  EA = 1;


  return 0;

}
/**
 * ʼ㸨
 */
unsigned char modBaudE(long vBaudrate) {
  return   ( \
    (vBaudrate==9600)   ?  8   : \
    (vBaudrate==19200)  ?  9   : \
    (vBaudrate==38400)  ?  10  : \
    (vBaudrate==57600)  ?  10  : \
    (vBaudrate==115200) ?  11  : \
   0);
}


unsigned char modBaudM(long vBaudrate) {
  return    ( \
    (vBaudrate==9600)   ?  59  : \
    (vBaudrate==19200)  ?  59  : \
    (vBaudrate==38400)  ?  59  : \
    (vBaudrate==57600)  ?  216 : \
    (vBaudrate==115200) ?  216 : \
   0);
}

/**
 * ͷӿ
 */
/**
 * ʽ
 * Flag(0x7E) Flag(0x7E) LEN(1byte, unsigned) data
 */
int8 modUartSendPkt(uint8 vUartNumber, uint8 vSize, const uint8 *vPacket) {

  if (vUartNumber != 0) {
    return -1;
  }
  unsigned char i;
  if (modUart0GetTxFreeBufferSize()<vSize+3) {
    return -2;
  }
  modUart0PushTxFIFO(0x7e);
  modUart0PushTxFIFO(0x7e);
  modUart0PushTxFIFO(vSize);
  for (i =0;i<vSize;i++) {
    modUart0PushTxFIFO(vPacket[i]);
  }

  if (gUart0SendingFlag==0) {
    U0DBUF = modUart0PopTxFIFO();
    gUart0SendingFlag = 1;
  }

  return 0;
}

/**
 * headerpacketpackettailer
 */
/**
 * ʽ
 * Flag(0x7E) Flag(0x7E) LEN(1byte, unsigned) Part1 Part2
 */
int8 modUartSendPktAndMore(uint8 vUartNumber, uint8 vPart1Size, const uint8 *vPart1, uint8 vPart2Size, const uint8 *vPart2) {
  uint8 i;
  uint16 iLen;

  if (vUartNumber != 0) {
    return -1;
  }
  iLen = vPart1Size+vPart2Size;
  if (iLen > 255) {
    return -3;
  }
  if (modUart0GetTxFreeBufferSize()<iLen+3) {
    return -2;
  }
  modUart0PushTxFIFO(0x7e);
  modUart0PushTxFIFO(0x7e);
  modUart0PushTxFIFO(iLen);
  for (i =0;i<vPart1Size;i++) {
    modUart0PushTxFIFO(vPart1[i]);
  }
  for (i =0;i<vPart2Size;i++) {
    modUart0PushTxFIFO(vPart2[i]);
  }

  if (gUart0SendingFlag==0) {
    U0DBUF = modUart0PopTxFIFO();
    gUart0SendingFlag = 1;
  }

  return 0;
}


/**
 * ȡײʵֵ֧飨ûݣ
 */
uint8 modUartGetMaxPduSize() {
  return MOD_UART_MAX_PDU_SIZE;
}

/**
 * շӿ
 */

/**
 * ýշصӿ
 * appģڸúʵֶԷĴ
 */

int8 modUartSetRxCallback(modUartRxCallbackFunction* vFunction) {
  mUartRxUsrCallBackFunction = vFunction;
  return 0;
}

/**
 * uart send buffer mechanism:
 * A gUart0TxStartPointer points to the position which next data byte to be write to buffer.
 * A gUart0TxEndPointer points to the position which next data byte to be read and send to uart.
 * When gUart0TxStartPointer=gUart0TxEndPointer-1, means writing is catching up reading, no more
 * data should be written.
 * So the total empty buffer size is UART_TX_BUFFER_LEN-1.
 *
 */


/**
 * check Buffer free space
 */
uint16 modUart0GetTxFreeBufferSize() {
  if (gUart0TxStartPointer == gUart0TxEndPointer) {
    //return UART_TX_BUFFER_LEN;
	return UART_TX_BUFFER_LEN-1;
  }else if (gUart0TxStartPointer<gUart0TxEndPointer) {
    /**
	 * debug: 2014.03.04 
    uint16 iTemp = gUart0TxEndPointer - gUart0TxStartPointer;
	*/
	uint16 iTemp = gUart0TxEndPointer - gUart0TxStartPointer-1;
    return iTemp;
  }else {
    //uint16 iTemp =  UART_TX_BUFFER_LEN - (gUart0TxStartPointer - gUart0TxEndPointer);
	uint16 iTemp =  UART_TX_BUFFER_LEN - (gUart0TxStartPointer - gUart0TxEndPointer)-1;
    return iTemp;
  }
}

/**
 * write uart tx FIFO
 */
int8 modUart0PushTxFIFO(unsigned char vData) {
  uint16 iNewStartPointer = gUart0TxStartPointer + 1;
  if (iNewStartPointer >=UART_TX_BUFFER_LEN) {
    iNewStartPointer = 0;
  }
  if (iNewStartPointer == gUart0TxEndPointer) { //faster than hardware TX
    return -1; //drop new data, return error
  }
  gUart0TxBuffer[gUart0TxStartPointer] = vData;
  gUart0TxStartPointer=iNewStartPointer;
  return 0;
}

/**
 * read uart tx FIFO
 */
unsigned char modUart0PopTxFIFO() {
  unsigned char ret;
  if (gUart0TxEndPointer == gUart0TxStartPointer) {
    return 0;
  }
  ret = gUart0TxBuffer[gUart0TxEndPointer];
  //
  gUart0TxEndPointer ++;

  if (gUart0TxEndPointer >=UART_TX_BUFFER_LEN) {
    gUart0TxEndPointer = 0;
  }
  return ret;
}


/**
 * check Buffer free space
 */
uint16 modUart0GetRxFreeBufferSize() {
  if (gUart0RxStartPointer == gUart0RxEndPointer) {
    return UART_RX_BUFFER_LEN-1;
  }else if (gUart0RxStartPointer<gUart0RxEndPointer) {
    uint16 iTemp = gUart0RxEndPointer - gUart0RxStartPointer-1;
    return iTemp;
  }else {
    uint16 iTemp =  UART_RX_BUFFER_LEN - (gUart0RxStartPointer - gUart0RxEndPointer)-1;
    return iTemp;
  }
}

/**
 * write uart RX FIFO
 */
int8 modUart0PushRxFIFO(unsigned char vData) {
  uint16 iNewStartPointer = gUart0RxStartPointer + 1;
  if (iNewStartPointer >=UART_RX_BUFFER_LEN) {
    iNewStartPointer = 0;
  }
  if (iNewStartPointer == gUart0RxEndPointer) { //faster than hardware TX
    return -1; //drop new data, return error
  }
  gUart0RxBuffer[gUart0RxStartPointer] = vData;
  gUart0RxStartPointer=iNewStartPointer;
  return 0;
}

/**
 * read uart RX FIFO
 */
unsigned char modUart0PopRxFIFO() {
  unsigned char ret;
  if (gUart0RxEndPointer == gUart0RxStartPointer) {
    return 0;
  }
  ret = gUart0RxBuffer[gUart0RxEndPointer];
  //
  gUart0RxEndPointer ++;

  if (gUart0RxEndPointer >=UART_RX_BUFFER_LEN) {
    gUart0RxEndPointer = 0;
  }
  return ret;
}



/**
 * Active Uart TX interrupt service routine
 */
//notice: call "U0DBUF = uart0PopTxFIFO();" not work.
void modUart0TxIrqRoutine() {
  UTX0IF = 0;
  if (gUart0TxEndPointer != gUart0TxStartPointer) {
    U0DBUF = gUart0TxBuffer[gUart0TxEndPointer];
    gUart0TxEndPointer ++;

    if (gUart0TxEndPointer >=UART_TX_BUFFER_LEN) {
      gUart0TxEndPointer = 0;
    }
  }else{
    gUart0SendingFlag = 0;
  }

}


/**
 * Active Uart RX interrupt service routine
 */
#define MOD_UART_RX_FLAG_IDLE 0
#define MOD_UART_RX_FLAG_SDF1 1
#define MOD_UART_RX_FLAG_SDF2 2
#define MOD_UART_RX_FLAG_LEN  3
#define MOD_UART_RX_FLAG_DATA 4
static uint8 gModUart0RxFlag=0;
static uint8 gModUart0RxDataRemain=0;
static uint8 gModUart0RxPktLen=0;

void modUart0RxIrqRoutine() {
  URX0IF = 0;
  uint8 iInData;
  iInData = U0DBUF;
  //
  switch (gModUart0RxFlag) {
  case MOD_UART_RX_FLAG_IDLE:
    if (iInData == 0x7e) {
      gModUart0RxFlag = MOD_UART_RX_FLAG_SDF1;
    }//else ignore
    break;
  case MOD_UART_RX_FLAG_SDF1:
    if (iInData == 0x7e) {
      gModUart0RxFlag = MOD_UART_RX_FLAG_SDF2;
    }else {//lost sync
      gModUart0RxFlag = MOD_UART_RX_FLAG_IDLE;
    }
    break;
  case MOD_UART_RX_FLAG_SDF2:
    //iInData is packet len
    gModUart0RxDataRemain = iInData;
    gModUart0RxFlag = MOD_UART_RX_FLAG_DATA;
    gModUart0RxPktLen= iInData;
    //gUart0RxStartPointer = 0;  //2014.04.06 modification
    //only reset gUart0RxStartPointer when rest buffer not enough for the new packet
    if ((gUart0RxStartPointer+gModUart0RxPktLen)>UART_RX_BUFFER_LEN) {
      gUart0RxStartPointer = 0;
    }
    break;
  case MOD_UART_RX_FLAG_DATA:
    gUart0RxBuffer[gUart0RxStartPointer] = iInData;
    gUart0RxStartPointer ++;
    gModUart0RxDataRemain --;
    if (gModUart0RxDataRemain == 0) {
      //receive one whole packet
      //call user routine
      if(mUartRxUsrCallBackFunction) {
        //mUartRxUsrCallBackFunction(0, gModUart0RxPktLen, gUart0RxBuffer); //2014.04.06 modification
        mUartRxUsrCallBackFunction(0, gModUart0RxPktLen, gUart0RxBuffer+gUart0RxStartPointer-gModUart0RxPktLen);
      }
      //
      gModUart0RxFlag = MOD_UART_RX_FLAG_IDLE;
    }
    break;

  }


}





/**
 * Uart interrupt service routine
 */
#pragma vector=UTX0_VECTOR
__interrupt static void Uart0Tx_IRQ(void) {
  EA=0;
  modUart0TxIrqRoutine();
  EA=1;
}

#pragma vector=URX0_VECTOR
__interrupt static void Uart0Rx_IRQ(void) {
  EA=0;
  modUart0RxIrqRoutine();
  EA=1;
}


